package org.vostok.management.test.web.partA;

import java.util.logging.Level;
import java.util.logging.Logger;
import javax.annotation.PostConstruct;
import javax.annotation.Resource;
import javax.management.ObjectName;
import org.vostok.management.annotations.JndiDependencies;
import org.vostok.management.annotations.LocalDependencies;
import org.vostok.management.annotations.MBean;
import org.vostok.management.mbean.MBeanFactory;

/**
 * This bean take a string as parameter, in the form of an operation, then return result. Part of the VostokMbeanSample pack. it use beans "Extract"
 *
 * @author levreau.jerome@vostoksystem.eu
 */
@MBean
@JndiDependencies("bean:org.vostok.management.test.web.partB=ArithmeticService")
@LocalDependencies(Extract.class)
public class Calculator {

	@Resource
	Extract extract;
	@Resource
	TrimMBean trim;

	public String operation(String str) throws BadFormatException {

		Extract.Item item = extract.convert(trim.trim(str));

		String method;
		switch (item.op) {
			case ADD:
				method = "add";
				break;
			case SUB:
				method = "sub";
				break;
			case DIV:
				method = "div";
				break;
			case MUL:
				method = "mul";
				break;
			default:
				throw new BadFormatException();
		}

		// as current version, with external mbean use of reflexion is required

		// get the mbean server from the factory, (this sample use default factory name)
		try {
			Object o = MBeanFactory.getFactory().getMbeanServer().invoke(
				new ObjectName("bean:org.vostok.management.test.web.partB=ArithmeticService"),
				method,
				new Object[]{item.a, item.b},
				new String[]{"float", "float"});

			return o.toString();

		} catch (Exception e) {
			Logger.getLogger(Calculator.class.getName()).log(Level.SEVERE, null, e);
		}

		return null;
	}

	@PostConstruct
	public void loaded() {
		Logger.getLogger(Calculator.class.getName()).log(Level.INFO, "Calculator started...");
	}
}
